/****************************************************************************
** $Id: rs_filtercxf.cpp,v 1.6 2003/09/15 12:58:36 andrew Exp $
**
** Copyright (C) 2001-2003 RibbonSoft. All rights reserved.
**
** This file is part of the qcadlib Library project.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding valid qcadlib Professional Edition licenses may use 
** this file in accordance with the qcadlib Commercial License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.ribbonsoft.com for further details.
**
** Contact info@ribbonsoft.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/


#include "rs_filtercxf.h"

#include <iostream>

#include "rs_regexp.h"
#include "rs_font.h"
#include "rs_utility.h"
#include "rs_system.h"


/**
 * Default constructor.
 *
 * @param graphic The graphic in which the entities from the file
 * will be created or the graphics from which the entities are
 * taken to be stored in a file.
 */
RS_FilterCXF::RS_FilterCXF(RS_Graphic* graphic)
        :RS_FilterInterface(graphic) {

    RS_DEBUG->print("Setting up CXF filter...");

    addImportExtension("cxf");
    addExportExtension("cxf");
}



/**
 * Implementation of the method used for RS_Import to communicate
 * with this filter.
 */
void RS_FilterCXF::fileImport(const RS_String& file) {
    RS_DEBUG->print("CXF Filter: importing file '%s'...", file.latin1());

    bool success = false;

    // Load font file as we normally do, but the font doesn't own the
    //  letters (we'll add them to the graphic instead. Hence 'false').
    RS_Font font(file, false);
    font.loadFont();

    graphic->addVariable("Names",
                         font.getNames().join(","), 0);
    graphic->addVariable("LetterSpacing", font.getLetterSpacing(), 0);
    graphic->addVariable("WordSpacing", font.getWordSpacing(), 0);
    graphic->addVariable("LineSpacingFactor", font.getLineSpacingFactor(), 0);
    graphic->addVariable("Authors", font.getAuthors().join(","), 0);
    if (!font.getEncoding().isEmpty()) {
        graphic->addVariable("Encoding", font.getEncoding(), 0);
    }

	RS_BlockList* letterList = font.getLetterList();
    for (uint i=0; i<font.countLetters(); ++i) {
        RS_Block* ch = font.letterAt(i);

        RS_String uCode;
        uCode.setNum(ch->getName().at(0).unicode(), 16);
        while (uCode.length()<4) {
            uCode="0"+uCode;
        }
        //ch->setName("[" + uCode + "] " + ch->getName());
		letterList->rename(ch, RS_String("[%1]").arg(ch->getName()));

        graphic->addBlock(ch, false);
        ch->reparent(graphic);
    }

    graphic->addBlockNotification();

    if (success!=true) {
        RS_DEBUG->print(RS_Debug::D_WARNING,
                        "Cannot open CXF file '%s'.", file.latin1());
    }
}



/**
 * Implementation of the method used for RS_Export to communicate
 * with this filter.
 *
 * @param file Full path to the CXF file that will be written.
 */
void RS_FilterCXF::fileExport(const RS_String& file) {

    RS_DEBUG->print("CXF Filter: exporting file '%s'...", file.latin1());

    std::ofstream fout;
    fout.open(file);

    // header:
    fout << "# Format:            QCad II Font\n";
    fout << "# Creator:           "
    << RS_SYSTEM->getAppName().local8Bit() << "\n";
    fout << "# Version:           "
    << RS_SYSTEM->getAppVersion().local8Bit() << "\n";
    RS_StringList names =
        RS_StringList::split(',', graphic->getVariableString("Names", ""));
    for (RS_StringList::Iterator it = names.begin(); it!=names.end(); ++it) {
        fout << "# Name:              " << (*it).local8Bit() << "\n";
    }
    if (!graphic->getVariableString("Encoding", "").isEmpty()) {
        fout << "# Encoding:          "
        << graphic->getVariableString("Encoding", "").local8Bit() << "\n";
    }
    fout << "# LetterSpacing:     "
    << graphic->getVariableDouble("LetterSpacing", 3.0) << "\n";
    fout << "# WordSpacing:       "
    << graphic->getVariableDouble("WordSpacing", 6.75) << "\n";
    fout << "# LineSpacingFactor: "
    << graphic->getVariableDouble("LineSpacingFactor", 1.0) << "\n";
    RS_StringList authors =
        RS_StringList::split(',', graphic->getVariableString("Authors", ""));
    for (RS_StringList::Iterator it = authors.begin(); it!=authors.end(); ++it) {
        fout << "# Author:            " << (*it).local8Bit() << "\n";
    }

    // iterate through blocks (=letters of font)
    for (uint i=0; i<graphic->countBlocks(); ++i) {
        RS_Block* blk = graphic->blockAt(i);

        fout << "\n" << blk->getName().local8Bit() << "\n";

        // iterate through entities of this letter:
        for (RS_Entity* e=blk->firstEntity(RS2::ResolveNone);
                e!=NULL;
                e=blk->nextEntity(RS2::ResolveNone)) {

            // lines:
            if (e->rtti()==RS2::EntityLine) {
                RS_Line* l = (RS_Line*)e;

                fout << "L ";
                stream(fout, l->getStartpoint().x);
                fout << ",";
                stream(fout, l->getStartpoint().y);
                fout << ",";
                stream(fout, l->getEndpoint().x);
                fout << ",";
                stream(fout, l->getEndpoint().y);
                fout << "\n";
            }

            // arcs:
            else if (e->rtti()==RS2::EntityArc) {
                RS_Arc* a = (RS_Arc*)e;

                if (!a->isReversed()) {
                    fout << "A ";
                } else {
                    fout << "AR ";
                }
                stream(fout, a->getCenter().x);
                fout << ",";
                stream(fout, a->getCenter().y);
                fout << ",";
                stream(fout, a->getRadius());
                fout << ",";
                stream(fout, a->getAngle1()*ARAD);
                fout << ",";
                stream(fout, a->getAngle2()*ARAD);
                fout << "\n";
            }
            // Ignore entities other than arcs / lines
            else {}
        }
    }
    fout.close();
}



/**
 * Streams a double value to the gien stream cutting away trailing 0's.
 *
 * @param value A double value. e.g. 2.700000
 */
void RS_FilterCXF::stream(std::ofstream& fs, double value) {
    fs << (const char*)RS_Utility::doubleToString(value);
}

// EOF

